#!/usr/bin/env python
# coding: utf-8

usage = '''
%prog [options] <team-name>
'''

# TODO: Make sure info is getting included, if relevant, for:
#   -psb
#   -openchrome
#   libxcb
#   mdetect
#   libpixman

# TODO: Add an OmitPackages to remove packages from display
#   -amd
#   -radeon

# TODO: Include extra urls
#   http://xlibs.freedesktop.org/xkbdesc/

# TODO: Include from nvidia (do I already have this implemented somewhere?)
#   ftp://download.nvidia.com/XFree86/Linux-x86/latest.txt

import os
import sys
import apt
import json
import urllib
from datetime import datetime
from optparse import OptionParser

from mako.template import Template
from mako.lookup import TemplateLookup

from xdiagnose.x_pkg_names import debpkg_to_fdopkg

def read_json(filename):
    file = open(filename, 'r')
    data = json.load(file)
    file.close()
    return data

# Strip epoch from version number
def non_epoch_version(version):
    try:
        return version.split(':', 1)[1]
    except IndexError:
        return version

def compare_versions(versionA, versionB):
    if versionA is None or versionA == '':
        return -1
    if versionB is None or versionB == '':
        return 1
    return apt.VersionCompare(non_epoch_version(versionA), non_epoch_version(versionB))

if __name__ == "__main__":
    root_dir = os.path.realpath(
        os.path.join(os.path.dirname(__file__), ".."))

    parser = OptionParser(usage=usage)
    parser.add_option(
        '-d', '--debug',
        action='store_true', dest='DEBUG', default=False,
        help='Enable debugging output')
    parser.add_option(
        '-j', '--json-data-path',
        action='store', dest='json_data_path',
        default=os.path.join(root_dir, 'reports', 'ubuntu-x-swat'),
        help='Path to JSON data files'
        )
    parser.add_option(
        '-t', '--templates-path',
        action='store', dest='templates_path',
        default=os.path.join(root_dir, 'templates'),
        help='Path to template files'
        )
    (opt, args) = parser.parse_args()

    if len(args) < 1:
        parser.print_help()
        sys.exit(1)
    team_name=args[0]

    upstream_name = 'upstream-%s' %('xorg') #TODO
    series_names = [
        'debian-testing', 'debian-unstable', 'debian-experimental',
        'ubuntu-stable', 'ubuntu-devel',
        ]
    column_names = [
        'Package',
        upstream_name,
        ]
    column_names.extend(series_names)
    column_names.append('Notes')
    series = {}
    columns = {}
    for col in column_names:
        json_path = os.path.join(opt.json_data_path, '%s.json' %(col))
        if os.path.exists(json_path):
            series[col] = read_json(json_path)
            columns[col] = series[col].get('_header', None)
        else:
            columns[col] = None

    packages = []
    package_names = series['ubuntu-devel'].keys()
    package_names.sort()
    for package_name in package_names:
        if package_name[0] == '_':
            continue
        fdo_pkg = debpkg_to_fdopkg(package_name)
        upstream_release = {}
        upstream_releases = series[upstream_name].get(fdo_pkg, None)
        if not upstream_releases:
            upstream_releases = series[upstream_name].get(package_name, None)
        if upstream_releases:
            upstream_release = upstream_releases[len(upstream_releases)-1]

        # TODO: Turn into a class?
        # TODO: Provide option for setting the notes url
        record = {
            'Package': package_name,
            'Notes': {
                'text': None,
                'wiki_url': 'http://wiki.ubuntu.com/X/PackageNotes',
                },
            upstream_name: {
                'version': upstream_release.get('version', None),
                'dsc': None,
                'url': upstream_release.get('url', None),
                'vcs': upstream_release.get('vcs', None),
                'status': 'black',
                },
            }
        for d in series_names:
            distro_name = d.split('-')[0]
            record[d] = {
                'version': None,
                'dsc': None,
                'url': None,
                'vcs': None,
                'status': None,
                }

            series_pkg = series[d].get(package_name, None)
            if series_pkg is None:
                continue

            # TODO: 'rel' is a horible variable name
            rel = series_pkg.get('Release', None)
            if rel is None:
                continue

            record[d] = rel
            record[d]['url'] = None # TODO
            record[d]['status'] = 'black'

            up_ver = upstream_release.get('version', '0')
            if distro_name == 'debian':
                package_version = non_epoch_version(rel['version'])
                if compare_versions(up_ver, package_version) > 0:
                    record[d]['status'] = 'needs-merge'
                prefix = package_name[0]
                if package_name[:3] == 'lib':
                    prefix = package_name[:4]
                section = 'main'
                # TODO: Some sort of lookup would be better here...
                if package_name[:15] == 'nvidia-graphics':
                    section = 'non-free'
                record[d]['dsc'] = 'http://ftp.debian.org/debian/pool/%s/%s/%s/%s_%s.dsc' %(
                    section, prefix, package_name, package_name, package_version)
                record[d]['url'] = 'http://packages.qa.debian.org/%s/%s.html' %(
                    prefix, package_name)

            elif distro_name == 'ubuntu':
                testing_ver = record['debian-testing']['version']
                unstable_ver = record['debian-unstable']['version']
                experimental_ver = record['debian-experimental']['version']

                if package_name == 'xtrans' and 1==2:
                    sys.stderr.write("%s (%s)\n" %(package_name, fdo_pkg))
                    sys.stderr.write("version: %s\n" %rel['version'])
                    sys.stderr.write("upstream: %s\n" %up_ver)
                    sys.stderr.write("testing: %s\n" %testing_ver)
                    sys.stderr.write("unstable: %s\n" %unstable_ver)
                    sys.stderr.write("experimental: %s\n" %experimental_ver)
                    sys.stderr.write("\n")

                if compare_versions(rel['version'], experimental_ver) < 0:
                    # if it's less than what's in experimental,
                    #   then if current version has 'ubuntu' in version string
                    #   it should be 'may-need-merge', otherwise 'may-need-sync'.
                    if 'ubuntu' in rel['version']:
                        record[d]['status'] = 'may-need-merge'
                    else:
                        record[d]['status'] = 'may-need-sync'

                elif compare_versions(rel['version'], unstable_ver) < 0:
                    # if it's less than what's in unstable,
                    #   if 'ubuntu' is in the version string, then 'needs-merge',
                    #   otherwise 'needs-sync'.
                    if 'ubuntu' in rel['version']:
                        record[d]['status'] = 'needs-merge'
                    else:
                        record[d]['status'] = 'needs-sync'

                elif compare_versions(rel['version'], testing_ver) < 0:
                    # if it's less than what's in 'testing',
                    #   if 'ubuntu' is in the version string, then 'needs-merge',
                    #   otherwise 'needs-sync'.
                    if 'ubuntu' in rel['version']:
                        record[d]['status'] = 'needs-merge'
                    else:
                        record[d]['status'] = 'needs-sync'

                # if it's less than the xorg version: debian-needs-merge
                elif compare_versions(up_ver, non_epoch_version(rel['version'])) > 0:
                    record[d]['status'] = 'debian-needs-merge'

                else:
                    record[d]['status'] = 'okay'

                record[d]['dsc'] = None
                record[d]['url'] = 'https://launchpad.net/ubuntu/%s/+source/%s' %(
                    rel['series'], package_name)

        packages.append(record)

    # TODO: Make template location configurable

    # TODO: Use proper tmp directory
    mako_lookup = TemplateLookup(directories=[opt.templates_path],
                                 module_directory='/tmp/mako_modules')
    t = mako_lookup.get_template("ubuntu+debian+upstream.html")
    print t.render(team=team_name,
                   column_names=column_names,
                   columns=columns,
                   data=packages,
                   today=datetime.strftime(datetime.now(), "%Y/%m/%d")
                   )
